/*
 * cc2520driver.h
 *
 *  Created on: Sep 18, 2010
 *      Author: Thomas Schmid
 */

#ifndef CC2520DRIVER_H_
#define CC2520DRIVER_H_

#include "config.h"

/**
 * Initialize the radio. This function has to be called only once
 * after powerup of the whole system. It initializes all the I/O
 * lines and puts the radio into LPM1 (everything initialized, but
 * oscillators are still turned off).
 */
void Radio_init();

/**
 * Turn on the radio receiver. The CC2520 now draws about 20 mA.
 *
 * return SUCCESS if the command succeeded.
 */
error_t RadioState_turnOn();

/**
 * Put the radio into Active mode, i.e., turn off the receiver
 * while keeping the oscillators running. This draws about 1 mA.
 * The advantage is that we can quickly switch back into receive
 * mode within about 192 us.
 *
 * return SUCCESS if the command succeeded.
 */
error_t RadioState_standby();

/**
 * Put the radio back into Low Power mode 1. In this mode, the radio
 * draws about 200 uA while retaining state. The oscillators are turned
 * off, but the voltage regulator is kept on. It takes at least 0.2 ms to
 * get back into receive mode.
 *
 * return SUCCESS if the command succeeded.
 */
error_t RadioState_turnOff();

/**
 * Shutdown the radio. This turns off the voltage regulator of the radio
 * and is the lowest power possible at 0.1 uA. It takes at least 0.3 ms
 * to get back into a receive mode (not including SPI initialization).
 *
 * Currently not implemented
 *
 * return SUCCESS if the command succeeded.
 */
//error_t RadioState_shutdown();

/**
 * Change the radio channel. Valid channels are 11 to 26, corresponding
 * to the 16 IEEE 802.15.4-2006 compliant channels.
 *
 * param c The radio channel to which we should switch
 * return SUCCESS if the command succeeded.
 */
error_t Radio_setRadioChannel(uint8_t c);

/**
 * Send a radio message.
 *
 * param seqn The sequence number of the message. The application developer
 *            has to assure that it is incrementing.
 * param panid The IEEE802.15.4 Personal Area Network ID. Can be used to
 *             filter messages belonging to a different network.
 * param saddr The source address of the message.
 * param daddr The destination address of the message.
 * param payload The data payload of the message.
 * param length Size in bytes of the payload.
 *
 * return SUCCESS if the command succeeded.
 */
error_t Radio_send(uint8_t seqn, uint16_t panid, uint16_t saddr, uint16_t daddr, uint8_t* payload, uint8_t length);

/**
 * This function has to be defined in your own code. It gets called when the
 * radio is done sending the message, and a new message can be sent. It is
 * kind of a callback function.
 *
 * param error SUCCESS if the message was successfully sent.
 */
void Radio_sendDone(error_t error);

/**
 * This function has to be defined in your own code. It indicates the reception
 * of a message by the radio.
 *
 * param seqn The sequence number of the message. The application developer
 *            has to assure that it is incrementing.
 * param panid The IEEE802.15.4 Personal Area Network ID. Can be used to
 *             filter messages belonging to a different network.
 * param saddr The source address of the message.
 * param daddr The destination address of the message.
 * param payload The data payload of the message.
 * param length Size in bytes of the payload.
 * param rssi The received signal strength measured during radio reception.
 */
void Radio_receive(uint8_t seqn, uint16_t panid, uint16_t saddr, uint16_t daddr, uint8_t* payload, uint8_t length, int8_t rssi);

#endif /* CC2520DRIVER_H_ */
